import { Mapper } from '@mobicoop/ddd-library';
import { Injectable } from '@nestjs/common';
import { Type } from './core/domain/username.types';
import { UsernameEntity } from './core/domain/username.entity';
import {
  UsernameReadModel,
  UsernameWriteModel,
} from './infrastructure/username.repository';
import { UsernameResponseDto } from './interface/dtos/username.response.dto';

/**
 * Mapper constructs objects that are used in different layers:
 * Record is an object that is stored in a database,
 * Entity is an object that is used in application domain layer,
 * and a ResponseDTO is an object returned to a user (usually as json).
 */

@Injectable()
export class UsernameMapper
  implements
    Mapper<
      UsernameEntity,
      UsernameReadModel,
      UsernameWriteModel,
      UsernameResponseDto
    >
{
  toPersistence = (entity: UsernameEntity): UsernameWriteModel => {
    const copy = entity.getProps();
    const record: UsernameWriteModel = {
      authUuid: copy.userId,
      username: copy.name,
      type: copy.type,
    };
    return record;
  };

  toDomain = (record: UsernameReadModel): UsernameEntity => {
    const entity = new UsernameEntity({
      id: record.username,
      createdAt: new Date(record.createdAt),
      updatedAt: new Date(record.updatedAt),
      props: {
        name: record.username,
        type: record.type as Type,
        userId: record.authUuid,
      },
    });
    return entity;
  };

  toResponse = (entity: UsernameEntity): UsernameResponseDto =>
    new UsernameResponseDto(entity);
}
